"""
This is the tool for handling the keywords

$Id: SmartKwManagerTool.py 7 2007-12-28 15:34:04Z gael.le-mignot $
"""

from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager

from Products.CMFCore.utils import UniqueObject, getToolByName


ID = 'portal_smart_keyword_manager'
TITLE = 'Smart Keyword Manager'
META_TYPE = 'SmartKwManagerTool'


class SmartKwManagerTool(UniqueObject, SimpleItem, PropertyManager):
    """
    This is the tool for handling the keywords
    """
    id = ID
    title = TITLE
    meta_type = META_TYPE

    ## Define properties
    _properties = ({'id': 'separator',
                    'label': 'Separator',
                    'mode': 'w',
                    'type': 'string'},
                   {'id': 'b_size',
                    'label': 'Default batch size',
                    'mode': 'w',
                    'type': 'int'},
                   )

    separator = ";"
    b_size = 50

    manage_options = (PropertyManager.manage_options
                      + SimpleItem.manage_options)

    ## Security shortcuts
    security = ClassSecurityInfo()

    security.declareProtected("Manage portal", "modifyKeywords")
    def modifyKeywords(self, uid, add = [], delete = []):
        """
        Modify keywords of object
        """
        # Shortcut out of dummy calls
        if not add and not delete:
            return
        
        # Get object from catalog
        catalog = getToolByName(self, "portal_catalog")
        brains = catalog(UID = uid)
        if len(brains) != 1:
            # Oh, oh, what ?
            return
        brain = brains[0]

        # Compute lists of keywords
        old = tuple(brain.Subject)
        delete = set(delete)
        new = [ kw for kw in old if not kw in delete ]
        existing = set(new)
        for kw in add:
            if not kw in existing:
                new.append(kw)
        new = tuple(new)

        # If anything change, write it
        if new != old:
            obj = brain.getObject()
            obj.setSubject(new)
            obj.reindexObject()

    security.declarePublic("getAllKeywords")
    def getAllKeywords(self):
        """
        Return sorted list of tags
        """
        catalog = getToolByName(self, "portal_catalog")
        keywords = list(catalog.uniqueValuesFor('Subject'))
        keywords.sort()
        return keywords

InitializeClass(SmartKwManagerTool)
